package com.nisum.hyd.service;

import org.apache.log4j.Logger;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.Row;
import org.apache.spark.sql.SparkSession;
import org.apache.spark.sql.functions;
import org.apache.spark.storage.StorageLevel;
import org.springframework.stereotype.Service;

@Service
public class SparkKafkaProducer {
    private static final Logger LOGGER = Logger.getLogger(SparkKafkaProducer.class);
    public static final String MONGODB_DOCUMENT_COLLECTION = "mongodb://127.0.0.1/usersdb";
    private static final String KafkaBrokerEndpoint = "localhost:9092";
    private static final String MONGODB_DATASOURCE = "com.mongodb.spark.sql.DefaultSource";
    private static String KafkaTopic = "UserWithCardDetails";

    public String getUserAndCardDataDetailsPushToKafka() {
        LOGGER.info("getUserCardDataAndPushToKafka() calling");
        SparkSession sparkSession = SparkSession.builder()
                .master("local[*]")
                .appName("MongoSparkConnector")
                .config("spark.mongodb.input.uri", MONGODB_DOCUMENT_COLLECTION)
                .config("spark.mongodb.output.uri", MONGODB_DOCUMENT_COLLECTION)
                .getOrCreate();
        Dataset<Row> usersInfo = sparkSession.read().format(MONGODB_DATASOURCE)
                .option("database", "usersdb").
                        option("collection", "userinfo").load();
        usersInfo.toDF().show();
        Dataset<Row> userCardInfo = sparkSession.read().format(MONGODB_DATASOURCE)
                .option("database", "usersdb").
                        option("collection", "usercardinfo").load();
        userCardInfo.toDF().show();
        Dataset<Row> userAndCardDetails = usersInfo
                .join(userCardInfo, usersInfo.colRegex("_id")
                        .equalTo(userCardInfo.colRegex("userId")), "inner");
        userAndCardDetails.show();
        sendToKafka(userAndCardDetails);
        return "Data Pushed to Kafka Successfully";
    }

    public void sendToKafka(Dataset<Row> userCardDetails) {
        Dataset<Row> KafkaPublish = userCardDetails
                .withColumn("value", functions.to_json(functions.struct(functions.col("userId"),
                        functions.col("userName"), functions.col("regMobileNo"),
                        functions.col("gender"),
                        functions.col("cardStatus"),
                        functions.col("cardNumber"),
                        functions.col("cardType"))))
                .alias("value").select("value")
                .persist(StorageLevel.MEMORY_AND_DISK());
        try {
            KafkaPublish.write().format("kafka")
                    .option("kafka.bootstrap.servers", KafkaBrokerEndpoint)
                    .option("topic", KafkaTopic)
                    .save();
        } catch (Exception e) {
            LOGGER.error("Error occurred in sendToKafka() while publishing to Kafka");
            e.getMessage();
        }
    }
}
